/// The type of an absolute point on a grid.
/// The actual origin point of these coordinates is irrelevant as long as the [`Grid`] instance treats them
/// consistently as Cartesian coordinates. For this reason, [`isize`] is used over [`usize`] for convenience
/// in the case that your use case involves negative coordinates.
pub type Pos = (isize, isize);

/// Inspection of the contents of a grid for the purposes of vision calculation.
/// Types implementing `Grid` represent a Cartesian grid of cells, each of which either blocks
/// vision ("is a wall") or does not.
pub trait Grid {
    /// Returns whether the provided position should be considered opaque.
    /// This method should return `true` for positions that are out of bounds to prevent infinite loops.
    fn is_wall(&self, pos: Pos) -> bool;

    /// This method is called when a certain position has been found to be reachable.
    /// Note that the library does not keep track of the size of the grid, and thus
    /// you may have to do bounds checking to explicitly ignore out-of-bounds positions.
    fn mark_visible(&mut self, pos: Pos);
}
